class Project
  include Mongoid::Document
  include Mongoid::Timestamps
  include OrbitModel::Status
  include MemberHelper
  include Slug

  belongs_to :project_type
  belongs_to :member_profile

  field :project_title, as: :slug_title, localize: true
  field :job_title, localize: true
  field :participator, localize: true
  field :unit, localize: true
  
  field :year, type: Integer
  field :language
  field :keywords
  field :abstract
  field :period_start_date, :type => Date
  field :period_end_date, :type => Date
  field :url
  field :note

  field :rss2_id
  field :create_user_id, :type => BSON::ObjectId
  field :update_user_id, :type => BSON::ObjectId

  paginates_per 10
  
  has_many :project_files, :autosave => true, :dependent => :destroy
  
  accepts_nested_attributes_for :project_files, :allow_destroy => true
  
  before_validation :add_http

  scope :sort_for_frontend, ->{ where(:is_hidden=>false).order_by(:year=>'desc', :period_start_date=>'desc') }

  def duration
    "#{self.period_start_date.strftime("%Y.%m") rescue ""} ~ #{self.period_end_date.strftime("%Y.%m") rescue ""}"
  end

  def self.get_plugin_datas_to_member(datas)

    fields_to_show = [
      "project_type",
      "year",
      "project_title",
      "participator",
      "job_title",
      "period",
      "unit"
    ]

    fields_to_remove = []

    pd_title = []

    fields_to_show.each do |t|
      if (self.fields[t].type.to_s == "String" || self.fields[t].type.to_s == "Object" rescue false)
        fields_to_remove << t if (datas.where(t.to_sym.ne => nil, t.to_sym.ne => "").count == 0 rescue false)
      else
        fields_to_remove << t if (datas.where(t.to_sym.ne => nil).count == 0 rescue false)
      end
      pd_title <<  {
          "plugin_data_title" => I18n.t("personal_project.#{t}")
        } if !fields_to_remove.include?(t)
    end

    fields_to_show = fields_to_show - fields_to_remove

    plugin_datas = datas.sort_for_frontend.collect.with_index do |p,index|

      pd_data = []
      fields_to_show.collect do |t|
        if t == "project_title"
          pd_data << { "data_title" => "<a href='#{OrbitHelper.url_to_plugin_show(p.to_param,'personal_project')}' target='_blank'>#{p.send(t)}" }
        elsif t == "period"

          if !p.send('period_start_date').nil? or !p.send('period_end_date').nil?
            date = (p.send('period_start_date').strftime('%Y.%m') rescue "")+' ~ '+(p.send('period_end_date').strftime('%Y.%m') rescue I18n.t('personal_project.up_to_today'))
          else
            date = ""
          end
          
          pd_data << { "data_title" => date }
        elsif t == "project_type"
          pd_data << {"data_title" => (p.project_type.title rescue "")}
        else
          pd_data << { "data_title" => p.send(t) }
        end
      end

      {
        "pd_datas" => pd_data,
        "type-sort" => (p.project_type.sort_position.to_i rescue 1000),
        "sort-index" => index
      }

    end
    plugin_datas = plugin_datas.sort_by{|pd| [pd["type-sort"], pd["sort-index"]]}
    return [pd_title,plugin_datas]
    
  end
    
  def get_plugin_data(fields_to_show)
    plugin_datas = []
    fields_to_show.each do |field|
      plugin_data = self.get_plugin_field_data(field) rescue nil
      next if plugin_data.blank? or plugin_data['value'].blank?
      plugin_datas << plugin_data
    end
    plugin_datas
  end

  def get_plugin_field_data(field)
    case field
    when "language"
      value = I18n.t(self.language) rescue ""
    when "project_type"
      value = self.project_type.title rescue ""
    when "period"
      value = self.duration
    when "abstract"
      value = nl2br(self.abstract)
    when "file"
      files = []
      self.project_files.each do |project_file|  
        url = project_file.file.url
        title = (project_file.title.blank? ? File.basename(project_file.file.path) : project_file.title)
        files << "<li><a href='#{url}'' target='_blank'>#{title}</li>"
      end
      value = files.join("")
    else
      value = self.send(field) rescue ""
    end

    value = (value =~ /\A#{URI::regexp(['http', 'https'])}\z/) ? "<a href='#{value}' target='blank'>#{value}</a>" : value
    
    {
      "key"=>field,
      "title_class"=>"project-#{field.gsub('_','-')}-field",
      "value_class"=>"project-#{field.gsub('_','-')}-value",
      "title"=>I18n.t('personal_project.'+field),
      "value"=>value
    }
  end
  
  protected

  def add_http
    unless self.url.blank? || self.url[/^http:\/\//] || self.url[/^https:\/\//]
      self.url = 'http://' + self.url
    end
  end
  
end